#include <avr/io.h>
#include <util/delay.h>

//Servo servo_01;  // create servo object to control a servo

int BatVoltage = 0;  // analog pin used to connect the potentiometer
int tempInt = 0;
int BatWarning = 0;
unsigned long comMillis;
int LEDstatus;
unsigned long shutDownMillis;
unsigned long LEDMillis;
unsigned long LEDonTime;

#define BAT_LED_GREEN              8
#define BAT_LED_YELLOW            12
#define BAT_LED_RED               13
#define POWER_PIN                  0
#define BAT_VOLTAGE_PIN           A0
#define BAT_MIN_VOLTAGE_FOR_ON   300
#define LOW_BATT_VOLTAGE         290
#define HIGH_BATT_VOLTAGE        320
#define COM_CYCLE                  4
#define COM_SEND_TO_RPI            3
#define COM_RECEIVE_FROM_RPI       2
#define MAX_COM_MILLIS         1000L
#define SHUTDOWN_TIMER       100000L


void setup(){
  pinMode(BAT_LED_GREEN, OUTPUT);
  pinMode(BAT_LED_YELLOW, OUTPUT);
  pinMode(BAT_LED_RED, OUTPUT);
  pinMode(POWER_PIN, OUTPUT);

  pinMode(COM_SEND_TO_RPI, OUTPUT);
  pinMode(COM_CYCLE, INPUT);
  pinMode(COM_RECEIVE_FROM_RPI, INPUT);

  digitalWrite(POWER_PIN, LOW);

  LEDstatus = 0;
  BatWarning = 2;
  shutDownMillis = 0L;

  LEDMillis = millis();
}

int comTimeout(){
  unsigned long nowMillis;

  nowMillis = millis();

  if(nowMillis > comMillis + MAX_COM_MILLIS){
    digitalWrite(BAT_LED_YELLOW, HIGH);   // set the LED on
    return 1;
  }

  if(nowMillis < comMillis){
    digitalWrite(BAT_LED_YELLOW, HIGH);   // set the LED on
    return 2;
  }

  return 0;
}

void sendBatStatus(){
  tempInt = BatVoltage;
  if(BatWarning == 1){
    tempInt += 1024;
  }

  comMillis = millis();

//!!!!!!!!!!!!!!!
  digitalWrite(BAT_LED_RED, HIGH);   // set the LED on
  digitalWrite(BAT_LED_YELLOW, LOW);   // set the LED on
  digitalWrite(BAT_LED_GREEN, HIGH);   // set the LED on
//!!!!!!!!!!!!!!!

  digitalWrite(COM_SEND_TO_RPI, HIGH);
  while(digitalRead(COM_CYCLE) == HIGH && comTimeout() == 0);
  digitalWrite(COM_SEND_TO_RPI, LOW);
  while(digitalRead(COM_CYCLE) == LOW && comTimeout() == 0);


  if(tempInt >= 1024){
    digitalWrite(COM_SEND_TO_RPI, HIGH);
    tempInt -= 1024;
  }
  else{
    digitalWrite(COM_SEND_TO_RPI, LOW);
  }
  while(digitalRead(COM_CYCLE) == HIGH && comTimeout() == 0);
  while(digitalRead(COM_CYCLE) == LOW && comTimeout() == 0);

  if(tempInt >= 512){
    digitalWrite(COM_SEND_TO_RPI, HIGH);
    tempInt -= 512;
  }
  else{
    digitalWrite(COM_SEND_TO_RPI, LOW);
  }
  while(digitalRead(COM_CYCLE) == HIGH && comTimeout() == 0);
  while(digitalRead(COM_CYCLE) == LOW && comTimeout() == 0);

  if(tempInt >= 256){
    digitalWrite(COM_SEND_TO_RPI, HIGH);
    tempInt -= 256;
  }
  else{
    digitalWrite(COM_SEND_TO_RPI, LOW);
  }
  while(digitalRead(COM_CYCLE) == HIGH && comTimeout() == 0);
  while(digitalRead(COM_CYCLE) == LOW && comTimeout() == 0);

  if(tempInt >= 128){
    digitalWrite(COM_SEND_TO_RPI, HIGH);
    tempInt -= 128;
  }
  else{
    digitalWrite(COM_SEND_TO_RPI, LOW);
  }
  while(digitalRead(COM_CYCLE) == HIGH && comTimeout() == 0);
  while(digitalRead(COM_CYCLE) == LOW && comTimeout() == 0);

  if(tempInt >= 64){
    digitalWrite(COM_SEND_TO_RPI, HIGH);
    tempInt -= 64;
  }
  else{
    digitalWrite(COM_SEND_TO_RPI, LOW);
  }
  while(digitalRead(COM_CYCLE) == HIGH && comTimeout() == 0);
  while(digitalRead(COM_CYCLE) == LOW && comTimeout() == 0);

  if(tempInt >= 32){
    digitalWrite(COM_SEND_TO_RPI, HIGH);
    tempInt -= 32;
  }
  else{
    digitalWrite(COM_SEND_TO_RPI, LOW);
  }
  while(digitalRead(COM_CYCLE) == HIGH && comTimeout() == 0);
  while(digitalRead(COM_CYCLE) == LOW && comTimeout() == 0);

  if(tempInt >= 16){
    digitalWrite(COM_SEND_TO_RPI, HIGH);
    tempInt -= 16;
  }
  else{
    digitalWrite(COM_SEND_TO_RPI, LOW);
  }
  while(digitalRead(COM_CYCLE) == HIGH && comTimeout() == 0);
  while(digitalRead(COM_CYCLE) == LOW && comTimeout() == 0);

  if(tempInt >= 8){
    digitalWrite(COM_SEND_TO_RPI, HIGH);
    tempInt -= 8;
  }
  else{
    digitalWrite(COM_SEND_TO_RPI, LOW);
  }
  while(digitalRead(COM_CYCLE) == HIGH && comTimeout() == 0);
  while(digitalRead(COM_CYCLE) == LOW && comTimeout() == 0);

  if(tempInt >= 4){
    digitalWrite(COM_SEND_TO_RPI, HIGH);
    tempInt -= 4;
  }
  else{
    digitalWrite(COM_SEND_TO_RPI, LOW);
  }
  while(digitalRead(COM_CYCLE) == HIGH && comTimeout() == 0);
  while(digitalRead(COM_CYCLE) == LOW && comTimeout() == 0);

  if(tempInt >= 2){
    digitalWrite(COM_SEND_TO_RPI, HIGH);
    tempInt -= 2;
  }
  else{
    digitalWrite(COM_SEND_TO_RPI, LOW);
  }
  while(digitalRead(COM_CYCLE) == HIGH && comTimeout() == 0);
  while(digitalRead(COM_CYCLE) == LOW && comTimeout() == 0);

  if(tempInt >= 1){
    digitalWrite(COM_SEND_TO_RPI, HIGH);
    tempInt -= 1;
  }
  else{
    digitalWrite(COM_SEND_TO_RPI, LOW);
  }
  while(digitalRead(COM_CYCLE) == HIGH && comTimeout() == 0);
  while(digitalRead(COM_CYCLE) == LOW && comTimeout() == 0);


  digitalWrite(COM_SEND_TO_RPI, LOW);

//!!!!!!!!!!!!!!!
  digitalWrite(BAT_LED_RED, LOW);   // set the LED on
  digitalWrite(BAT_LED_YELLOW, LOW);   // set the LED on
  digitalWrite(BAT_LED_GREEN, LOW);   // set the LED on
//!!!!!!!!!!!!!!!

}

void loop(){

  BatVoltage = analogRead(BAT_VOLTAGE_PIN);

  if(BatVoltage < LOW_BATT_VOLTAGE){
    BatWarning = 1;
    if(shutDownMillis == 0 || millis() < shutDownMillis){
      shutDownMillis = millis();
    }
  }

  if(BatVoltage > BAT_MIN_VOLTAGE_FOR_ON){
    BatWarning = 0;
    shutDownMillis = 0;
    digitalWrite(POWER_PIN, HIGH);
  }

  if(shutDownMillis > 0){
    if(millis() > shutDownMillis + SHUTDOWN_TIMER){
      digitalWrite(POWER_PIN, LOW);
    }
  }

  if(digitalRead(COM_CYCLE) == HIGH){
    if(digitalRead(COM_RECEIVE_FROM_RPI) == LOW){
      sendBatStatus();
    }
  }

  if(BatWarning == 1){
    if(millis() - LEDMillis < 500){
      digitalWrite(BAT_LED_GREEN, HIGH);
      digitalWrite(BAT_LED_YELLOW, HIGH);
      digitalWrite(BAT_LED_RED, HIGH);
    }
    else{
      digitalWrite(BAT_LED_GREEN, LOW);
      digitalWrite(BAT_LED_YELLOW, LOW);
      digitalWrite(BAT_LED_RED, LOW);
    }
  }
  if(BatWarning == 2){
    digitalWrite(BAT_LED_GREEN, LOW);
    digitalWrite(BAT_LED_YELLOW, LOW);
    digitalWrite(BAT_LED_RED, LOW);

    if(millis() - LEDMillis > 300){
      digitalWrite(BAT_LED_RED, HIGH);
    }
    if(millis() - LEDMillis > 600){
      digitalWrite(BAT_LED_YELLOW, HIGH);
    }
    if(millis() - LEDMillis > 900){
      digitalWrite(BAT_LED_GREEN, HIGH);
    }
  }
  if(BatWarning == 0){
    LEDonTime = (unsigned long)(map(BatVoltage, LOW_BATT_VOLTAGE, HIGH_BATT_VOLTAGE, 0, 3000));
    if(LEDonTime >= 2000){
      if(LEDonTime - 2000 > millis() - LEDMillis){
        digitalWrite(BAT_LED_GREEN, HIGH);
        digitalWrite(BAT_LED_YELLOW, HIGH);
        digitalWrite(BAT_LED_RED, HIGH);
      }
      else{
        digitalWrite(BAT_LED_GREEN, LOW);
      }
    }

    if(LEDonTime < 2000 && LEDonTime >= 1000){
      if(LEDonTime - 1000 > millis() - LEDMillis){
        digitalWrite(BAT_LED_YELLOW, HIGH);
        digitalWrite(BAT_LED_RED, HIGH);
      }
      else{
        digitalWrite(BAT_LED_YELLOW, LOW);
        digitalWrite(BAT_LED_GREEN, LOW);
      }
    }

    if(LEDonTime < 1000){
      if(LEDonTime > millis() - LEDMillis){
        digitalWrite(BAT_LED_RED, HIGH);
      }
      else{
        digitalWrite(BAT_LED_YELLOW, LOW);
        digitalWrite(BAT_LED_GREEN, LOW);
        digitalWrite(BAT_LED_RED, LOW);
      }
    }
  }

  if(millis() - LEDMillis > 1000L){
    LEDMillis = millis();
  }


}



