//R7 gpio control  program
//www.RoboSpatium.de
//compile with: gcc R7C1.c -o R7C1 -I/usr/local/include -L/usr/local/lib -lwiringPi -lpthread -lm




#include <wiringPi.h>
#include <softPwm.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>


#define LIGHT_01                    23
#define LIGHT_02                    24
#define MOTOR_01                     1
#define MOTOR_02                     4
#define STEERING_01                  5
#define STEERING_02                  6
#define BAT_LOW                      3
#define BAT_MIDDLE                   2
#define BAT_HIGH                     0

#define ONLINE_SWITCH               21
#define SHUTDOWN_SWITCH             22


#define CYCLE_PAUSE               1000
#define MAX_DRIVE_LENGTH      10000000

#define MOTOR_CYCLE    80



char commandLine[1000];

static void usage(){
  printf("usage:\n");
  printf("R7C1 [actuator] [value]\n");
  printf("l - Light intensity (0 - 4)\n");
}

void BatLevel(){
  char charTemp[1000]="";
  FILE *BATFILE;

  if(!digitalRead(BAT_HIGH)){
    if((BATFILE=fopen("./BAT_HIGH.txt","wb"))==NULL){
      printf("Cant create BAT_HIGH.txt!\n");
      exit(1);
    }
    else{
      strcpy(charTemp, "Battery isn't HIGH");
      fwrite(&charTemp, strlen(charTemp), 1, BATFILE);
      fclose(BATFILE);
    }
  }

  if(!digitalRead(BAT_MIDDLE)){
    if((BATFILE=fopen("./BAT_MIDDLE.txt","wb"))==NULL){
      printf("Cant create BAT_MIDDLE.txt!\n");
      exit(1);
    }
    else{
      strcpy(charTemp, "Battery isn't MIDDLE");
      fwrite(&charTemp, strlen(charTemp), 1, BATFILE);
      fclose(BATFILE);
    }
  }

  if(!digitalRead(BAT_LOW)){
    if((BATFILE=fopen("./BAT_LOW.txt","wb"))==NULL){
      printf("Cant create BAT_LOW.txt!\n");
      exit(1);
    }
    else{
      strcpy(charTemp, "Battery isn't LOW");
      fwrite(&charTemp, strlen(charTemp), 1, BATFILE);
      fclose(BATFILE);
    }
  }
}

//######################################################################
//######################################################################
//######################################################################
int main(int argc, char **argv){
  int i;
  int key;
  unsigned char buff;
  int BatteryStatus = 0;
  int Verbose = 0;
  char charTemp[1000]="";
  FILE *BATFILE;



  if (wiringPiSetup () == -1)
    exit (1) ;


  for(i=1; i<argc; i++){
    printf("argv[%d]=%s ", i, argv[i]);
  }
  printf("\n");

  if(argc<2){
    usage();
    return(0);
  }

  if(argc>3){
    if(strcmp(argv[3], "v") == 0){
      Verbose = 1;
    }
    if(argc>4){
      if(atoi(argv[4]) > 1){
        Verbose = atoi(argv[4]);
      }
    }
  }



  printf ("\nInitializing GPIOs.\n");

  pinMode (ONLINE_SWITCH, INPUT);
  pullUpDnControl (ONLINE_SWITCH, PUD_UP) ;
  pinMode (SHUTDOWN_SWITCH, INPUT);
  pullUpDnControl (SHUTDOWN_SWITCH, PUD_UP) ;

  pinMode (BAT_LOW, INPUT);
  pullUpDnControl (BAT_LOW, PUD_OFF) ;
  pinMode (BAT_MIDDLE, INPUT);
  pullUpDnControl (BAT_MIDDLE, PUD_OFF) ;
  pinMode (BAT_HIGH, INPUT);
  pullUpDnControl (BAT_HIGH, PUD_OFF) ;


  pinMode (LIGHT_01, OUTPUT);
  pinMode (LIGHT_02, OUTPUT);

  pinMode (MOTOR_01, OUTPUT);
  pinMode (MOTOR_02, OUTPUT);

  softPwmCreate(MOTOR_01, 0, 200);
  softPwmCreate(MOTOR_02, 0, 200);

  pinMode (STEERING_01, OUTPUT);
  pinMode (STEERING_02, OUTPUT);

  digitalWrite(MOTOR_01, 0);
  digitalWrite(MOTOR_02, 0);
  digitalWrite(STEERING_01, 0);
  digitalWrite(STEERING_02, 0);



//  softPwmCreate(CAMERA_SERVO, 0, 200);


  if(strcmp(argv[1], "fr") == 0 || strcmp(argv[1], "br") == 0){//Turn right
    digitalWrite(STEERING_01, 0);
    digitalWrite(STEERING_02, 1);
    usleep(500000);
  }
  if(strcmp(argv[1], "fl") == 0 || strcmp(argv[1], "bl") == 0){//Turn left
    digitalWrite(STEERING_01, 1);
    digitalWrite(STEERING_02, 0);
    usleep(500000);
  }

  if(strcmp(argv[1], "f") == 0 || strcmp(argv[1], "fr") == 0 || strcmp(argv[1], "fl") == 0){//Driving forward
    if(atoi(argv[2]) > 0 && atoi(argv[2]) < MAX_DRIVE_LENGTH){
      softPwmWrite(MOTOR_01, MOTOR_CYCLE);
      softPwmWrite(MOTOR_02, 0);
      int steeringAdjust = 0;
      for(i=0; i<atoi(argv[2]); i+=10000){
        usleep(10000);
        BatLevel();
      }
      softPwmWrite(MOTOR_01, 0);
      softPwmWrite(MOTOR_02, 0);
    }
  }

  if(strcmp(argv[1], "b") == 0 || strcmp(argv[1], "br") == 0 || strcmp(argv[1], "bl") == 0){//Driving backward
    if(atoi(argv[2]) > 0 && atoi(argv[2]) < MAX_DRIVE_LENGTH){
      softPwmWrite(MOTOR_01, 0);
      softPwmWrite(MOTOR_02, MOTOR_CYCLE);
      int steeringAdjust = 0;
      for(i=0; i<atoi(argv[2]); i+=10000){
        usleep(10000);
        BatLevel();
      }
      softPwmWrite(MOTOR_01, 0);
      softPwmWrite(MOTOR_02, 0);
    }
  }


  //turn off steering
  digitalWrite(STEERING_01, 0);
  digitalWrite(STEERING_02, 0);




  if(strcmp(argv[1], "s") == 0){//Switches
    if(digitalRead(ONLINE_SWITCH) == 0){
      printf("ONLINE!\n");
    }
    else{
      printf("offline.\n");
    }
    if(digitalRead(SHUTDOWN_SWITCH) == 0){
      printf("SHUTDOWN!\n");
    }
    else{
      printf("no shutdown.\n");
    }
  }

  if(strcmp(argv[1], "h") == 0){//Headlight
    if(atoi(argv[2]) < 0 || atoi(argv[2]) > 4){
      usage();
      return(0);
    }
    printf("Switching light intensity to %s\n", argv[2]);
    if(strcmp(argv[2], "0") == 0){
      digitalWrite(LIGHT_01, 0);
      digitalWrite(LIGHT_02, 0);
    }
    if(strcmp(argv[2], "1") == 0){
      digitalWrite(LIGHT_01, 1);
      digitalWrite(LIGHT_02, 0);
    }
    if(strcmp(argv[2], "2") == 0){
      digitalWrite(LIGHT_01, 0);
      digitalWrite(LIGHT_02, 1);
    }
    if(strcmp(argv[2], "3") == 0){
      digitalWrite(LIGHT_01, 1);
      digitalWrite(LIGHT_02, 1);
    }
  }


  BatLevel();

  printf("...done!\n\n");

  usleep(50000);

  return (0);


}



